@@ -44,7 +44,6 @@ ZEND_API void zend_generator_restore_call_stack(zend_generator *generator) /* {{
 			ZEND_CALL_NUM_ARGS(call),
 			Z_PTR(call->This));
 		memcpy(((zval*)new_call) + ZEND_CALL_FRAME_SLOT, ((zval*)call) + ZEND_CALL_FRAME_SLOT, ZEND_CALL_NUM_ARGS(call) * sizeof(zval));
-		new_call->extra_named_params = call->extra_named_params;
 		new_call->prev_execute_data = prev_call;
 		prev_call = new_call;
 
@@ -95,20 +94,6 @@ ZEND_API zend_execute_data* zend_generator_freeze_call_stack(zend_execute_data *
 }
 /* }}} */
 
-static zend_execute_data* zend_generator_revert_call_stack(zend_execute_data *call)
-{
-	zend_execute_data *prev = NULL;
-
-	do {
-		zend_execute_data *next = call->prev_execute_data;
-		call->prev_execute_data = prev;
-		prev = call;
-		call = next;
-	} while (call);
-
-	return prev;
-}
-
 static void zend_generator_cleanup_unfinished_execution(
 		zend_generator *generator, zend_execute_data *execute_data, uint32_t catch_op_num) /* {{{ */
 {
@@ -130,7 +115,7 @@ static void zend_generator_cleanup_unfinished_execution(
 }
 /* }}} */
 
-ZEND_API void zend_generator_close(zend_generator *generator, zend_bool finished_execution) /* {{{ */
+ZEND_API void zend_generator_close(zend_generator *generator, bool finished_execution) /* {{{ */
 {
 	if (EXPECTED(generator->execute_data)) {
 		zend_execute_data *execute_data = generator->execute_data;
@@ -180,14 +165,14 @@ static void zend_generator_remove_child(zend_generator_node *node, zend_generato
 {
 	ZEND_ASSERT(node->children >= 1);
 	if (node->children == 1) {
-		node->child.single.child = NULL;
+		node->child.single = NULL;
 	} else {
 		HashTable *ht = node->child.ht;
 		zend_hash_index_del(ht, (zend_ulong) child);
 		if (node->children == 2) {
 			zend_generator *other_child;
 			ZEND_HASH_FOREACH_PTR(ht, other_child) {
-				node->child.single.child = other_child;
+				node->child.single = other_child;
 				break;
 			} ZEND_HASH_FOREACH_END();
 			zend_hash_destroy(ht);
@@ -241,6 +226,7 @@ static void zend_generator_dtor_storage(zend_object *object) /* {{{ */
 
 	if (EXPECTED(!ex) || EXPECTED(!(ex->func->op_array.fn_flags & ZEND_ACC_HAS_FINALLY_BLOCK))
 			|| CG(unclean_shutdown)) {
+		zend_generator_close(generator, 0);
 		return;
 	}
 
@@ -280,7 +266,7 @@ static void zend_generator_dtor_storage(zend_object *object) /* {{{ */
 
 			/* TODO: If we hit another yield inside try/finally,
 			 * should we also jump to the next finally block? */
-			return;
+			break;
 		} else if (op_num < try_catch->finally_end) {
 			zval *fast_call =
 				ZEND_CALL_VAR(ex, ex->func->op_array.opcodes[try_catch->finally_end].op1.var);
@@ -299,6 +285,8 @@ static void zend_generator_dtor_storage(zend_object *object) /* {{{ */
 
 		try_catch_offset--;
 	}
+
+	zend_generator_close(generator, 0);
 }
 /* }}} */
 
@@ -384,15 +372,6 @@ static HashTable *zend_generator_get_gc(zend_object *object, zval **table, int *
 		zend_get_gc_buffer_add_zval(gc_buffer, &extra_named_params);
 	}
 
-	if (UNEXPECTED(generator->frozen_call_stack)) {
-		/* The frozen stack is linked in reverse order */
-		zend_execute_data *call = zend_generator_revert_call_stack(generator->frozen_call_stack);
-		/* -1 required because we want the last run opcode, not the next to-be-run one. */
-		uint32_t op_num = execute_data->opline - op_array->opcodes - 1;
-		zend_unfinished_calls_gc(execute_data, call, op_num, gc_buffer);
-		zend_generator_revert_call_stack(call);
-	}
-
 	if (execute_data->opline != op_array->opcodes) {
 		uint32_t i, op_num = execute_data->opline - op_array->opcodes - 1;
 		for (i = 0; i < op_array->last_live_range; i++) {
@@ -479,23 +458,21 @@ static void zend_generator_throw_exception(zend_generator *generator, zval *exce
 {
 	zend_execute_data *original_execute_data = EG(current_execute_data);
 
-	/* Throw the exception in the context of the generator. Decrementing the opline
-	 * to pretend the exception happened during the YIELD opcode. */
-	EG(current_execute_data) = generator->execute_data;
-	generator->execute_data->opline--;
-
-	if (exception) {
-		zend_throw_exception_object(exception);
-	} else {
-		zend_rethrow_exception(EG(current_execute_data));
-	}
-
 	/* if we don't stop an array/iterator yield from, the exception will only reach the generator after the values were all iterated over */
 	if (UNEXPECTED(Z_TYPE(generator->values) != IS_UNDEF)) {
 		zval_ptr_dtor(&generator->values);
 		ZVAL_UNDEF(&generator->values);
 	}
 
+	/* Throw the exception in the context of the generator. Decrementing the opline
+	 * to pretend the exception happened during the YIELD opcode. */
+	EG(current_execute_data) = generator->execute_data;
+	generator->execute_data->opline--;
+	if (exception) {
+		zend_throw_exception_object(exception);
+	} else {
+		zend_rethrow_exception(EG(current_execute_data));
+	}
 	generator->execute_data->opline++;
 	EG(current_execute_data) = original_execute_data;
 }
@@ -505,13 +482,13 @@ static void zend_generator_add_child(zend_generator *generator, zend_generator *
 	zend_generator_node *node = &generator->node;
 
 	if (node->children == 0) {
-		node->child.single.child = child;
+		node->child.single = child;
 	} else {
 		if (node->children == 1) {
 			HashTable *ht = emalloc(sizeof(HashTable));
 			zend_hash_init(ht, 0, NULL, NULL, 0);
 			zend_hash_index_add_new_ptr(ht,
-				(zend_ulong) node->child.single.child, node->child.single.child);
+				(zend_ulong) node->child.single, node->child.single);
 			node->child.ht = ht;
 		}
 
@@ -550,7 +527,7 @@ ZEND_API zend_generator *zend_generator_update_root(zend_generator *generator)
 static zend_generator *get_new_root(zend_generator *generator, zend_generator *root)
 {
 	while (!root->execute_data && root->node.children == 1) {
-		root = root->node.child.single.child;
+		root = root->node.child.single;
 	}
 
 	if (root->execute_data) {
@@ -626,8 +603,6 @@ ZEND_API zend_generator *zend_generator_update_current(zend_generator *generator
 
 static zend_result zend_generator_get_next_delegated_value(zend_generator *generator) /* {{{ */
 {
-	--generator->execute_data->opline;
-
 	zval *value;
 	if (Z_TYPE(generator->values) == IS_ARRAY) {
 		HashTable *ht = Z_ARR(generator->values);
@@ -642,9 +617,6 @@ static zend_result zend_generator_get_next_delegated_value(zend_generator *gener
 
 			p = &ht->arData[pos];
 			value = &p->val;
-			if (Z_TYPE_P(value) == IS_INDIRECT) {
-				value = Z_INDIRECT_P(value);
-			}
 			pos++;
 		} while (Z_ISUNDEF_P(value));
 
@@ -665,17 +637,22 @@ static zend_result zend_generator_get_next_delegated_value(zend_generator *gener
 		if (iter->index++ > 0) {
 			iter->funcs->move_forward(iter);
 			if (UNEXPECTED(EG(exception) != NULL)) {
-				goto failure;
+				goto exception;
 			}
 		}
 
 		if (iter->funcs->valid(iter) == FAILURE) {
+			if (UNEXPECTED(EG(exception) != NULL)) {
+				goto exception;
+			}
 			/* reached end of iteration */
 			goto failure;
 		}
 
 		value = iter->funcs->get_current_data(iter);
-		if (UNEXPECTED(EG(exception) != NULL) || UNEXPECTED(!value)) {
+		if (UNEXPECTED(EG(exception) != NULL)) {
+			goto exception;
+		} else if (UNEXPECTED(!value)) {
 			goto failure;
 		}
 
@@ -687,21 +664,20 @@ static zend_result zend_generator_get_next_delegated_value(zend_generator *gener
 			iter->funcs->get_current_key(iter, &generator->key);
 			if (UNEXPECTED(EG(exception) != NULL)) {
 				ZVAL_UNDEF(&generator->key);
-				goto failure;
+				goto exception;
 			}
 		} else {
 			ZVAL_LONG(&generator->key, iter->index);
 		}
 	}
-
-	++generator->execute_data->opline;
 	return SUCCESS;
 
+exception:
+	zend_generator_throw_exception(generator, NULL);
+
 failure:
 	zval_ptr_dtor(&generator->values);
 	ZVAL_UNDEF(&generator->values);
-
-	++generator->execute_data->opline;
 	return FAILURE;
 }
 /* }}} */
@@ -730,97 +706,94 @@ try_again:
 	/* Drop the AT_FIRST_YIELD flag */
 	orig_generator->flags &= ~ZEND_GENERATOR_AT_FIRST_YIELD;
 
-	/* Backup executor globals */
-	zend_execute_data *original_execute_data = EG(current_execute_data);
-	uint32_t original_jit_trace_num = EG(jit_trace_num);
-
-	/* Set executor globals */
-	EG(current_execute_data) = generator->execute_data;
-	EG(jit_trace_num) = 0;
-
-	/* We want the backtrace to look as if the generator function was
-	 * called from whatever method we are current running (e.g. next()).
-	 * So we have to link generator call frame with caller call frame. */
-	if (generator == orig_generator) {
-		generator->execute_data->prev_execute_data = original_execute_data;
-	} else {
-		/* We need some execute_data placeholder in stacktrace to be replaced
-		 * by the real stack trace when needed */
-		generator->execute_data->prev_execute_data = &orig_generator->execute_fake;
-		orig_generator->execute_fake.prev_execute_data = original_execute_data;
-	}
-
-	/* Ensure this is run after executor_data swap to have a proper stack trace */
 	if (UNEXPECTED(!Z_ISUNDEF(generator->values))) {
 		if (EXPECTED(zend_generator_get_next_delegated_value(generator) == SUCCESS)) {
-			/* Restore executor globals */
-			EG(current_execute_data) = original_execute_data;
-			EG(jit_trace_num) = original_jit_trace_num;
-
 			orig_generator->flags &= ~ZEND_GENERATOR_DO_INIT;
 			return;
 		}
-		/* If there are no more deletegated values, resume the generator
+		/* If there are no more delegated values, resume the generator
 		 * after the "yield from" expression. */
 	}
 
-	if (UNEXPECTED(generator->frozen_call_stack)) {
-		/* Restore frozen call-stack */
-		zend_generator_restore_call_stack(generator);
-	}
+	{
+		/* Backup executor globals */
+		zend_execute_data *original_execute_data = EG(current_execute_data);
+		uint32_t original_jit_trace_num = EG(jit_trace_num);
 
-	/* Resume execution */
-	generator->flags |= ZEND_GENERATOR_CURRENTLY_RUNNING;
-	if (!ZEND_OBSERVER_ENABLED) {
-		zend_execute_ex(generator->execute_data);
-	} else {
-		zend_observer_generator_resume(generator->execute_data);
-		zend_execute_ex(generator->execute_data);
-		if (generator->execute_data) {
-			/* On the final return, this will be called from ZEND_GENERATOR_RETURN */
-			zend_observer_fcall_end(generator->execute_data, &generator->value);
-		}
-	}
-	generator->flags &= ~ZEND_GENERATOR_CURRENTLY_RUNNING;
+		/* Set executor globals */
+		EG(current_execute_data) = generator->execute_data;
+		EG(jit_trace_num) = 0;
 
-	generator->frozen_call_stack = NULL;
-	if (EXPECTED(generator->execute_data) &&
-		UNEXPECTED(generator->execute_data->call)) {
-		/* Frize call-stack */
-		generator->frozen_call_stack = zend_generator_freeze_call_stack(generator->execute_data);
-	}
-
-	/* Restore executor globals */
-	EG(current_execute_data) = original_execute_data;
-	EG(jit_trace_num) = original_jit_trace_num;
-
-	/* If an exception was thrown in the generator we have to internally
-	 * rethrow it in the parent scope.
-	 * In case we did yield from, the Exception must be rethrown into
-	 * its calling frame (see above in if (check_yield_from). */
-	if (UNEXPECTED(EG(exception) != NULL)) {
+		/* We want the backtrace to look as if the generator function was
+		 * called from whatever method we are current running (e.g. next()).
+		 * So we have to link generator call frame with caller call frame. */
 		if (generator == orig_generator) {
-			zend_generator_close(generator, 0);
-			if (!EG(current_execute_data)) {
-				zend_throw_exception_internal(NULL);
-			} else if (EG(current_execute_data)->func &&
-					ZEND_USER_CODE(EG(current_execute_data)->func->common.type)) {
-				zend_rethrow_exception(EG(current_execute_data));
-			}
+			generator->execute_data->prev_execute_data = original_execute_data;
 		} else {
+			/* We need some execute_data placeholder in stacktrace to be replaced
+			 * by the real stack trace when needed */
+			generator->execute_data->prev_execute_data = &orig_generator->execute_fake;
+			orig_generator->execute_fake.prev_execute_data = original_execute_data;
+		}
+
+		if (UNEXPECTED(generator->frozen_call_stack)) {
+			/* Restore frozen call-stack */
+			zend_generator_restore_call_stack(generator);
+		}
+
+		/* Resume execution */
+		generator->flags |= ZEND_GENERATOR_CURRENTLY_RUNNING;
+		if (!ZEND_OBSERVER_ENABLED) {
+			zend_execute_ex(generator->execute_data);
+		} else {
+			zend_observer_generator_resume(generator->execute_data);
+			zend_execute_ex(generator->execute_data);
+			if (generator->execute_data) {
+				/* On the final return, this will be called from ZEND_GENERATOR_RETURN */
+				zend_observer_fcall_end(generator->execute_data, &generator->value);
+			}
+		}
+		generator->flags &= ~ZEND_GENERATOR_CURRENTLY_RUNNING;
+
+		generator->frozen_call_stack = NULL;
+		if (EXPECTED(generator->execute_data) &&
+		    UNEXPECTED(generator->execute_data->call)) {
+			/* Frize call-stack */
+			generator->frozen_call_stack = zend_generator_freeze_call_stack(generator->execute_data);
+		}
+
+		/* Restore executor globals */
+		EG(current_execute_data) = original_execute_data;
+		EG(jit_trace_num) = original_jit_trace_num;
+
+		/* If an exception was thrown in the generator we have to internally
+		 * rethrow it in the parent scope.
+		 * In case we did yield from, the Exception must be rethrown into
+		 * its calling frame (see above in if (check_yield_from). */
+		if (UNEXPECTED(EG(exception) != NULL)) {
+			if (generator == orig_generator) {
+				zend_generator_close(generator, 0);
+				if (!EG(current_execute_data)) {
+					zend_throw_exception_internal(NULL);
+				} else if (EG(current_execute_data)->func &&
+						ZEND_USER_CODE(EG(current_execute_data)->func->common.type)) {
+					zend_rethrow_exception(EG(current_execute_data));
+				}
+			} else {
+				generator = zend_generator_get_current(orig_generator);
+				zend_generator_throw_exception(generator, NULL);
+				orig_generator->flags &= ~ZEND_GENERATOR_DO_INIT;
+				goto try_again;
+			}
+		}
+
+		/* yield from was used, try another resume. */
+		if (UNEXPECTED((generator != orig_generator && !Z_ISUNDEF(generator->retval)) || (generator->execute_data && (generator->execute_data->opline - 1)->opcode == ZEND_YIELD_FROM))) {
 			generator = zend_generator_get_current(orig_generator);
-			zend_generator_throw_exception(generator, NULL);
-			orig_generator->flags &= ~ZEND_GENERATOR_DO_INIT;
 			goto try_again;
 		}
 	}
 
-	/* yield from was used, try another resume. */
-	if (UNEXPECTED((generator != orig_generator && !Z_ISUNDEF(generator->retval)) || (generator->execute_data && (generator->execute_data->opline - 1)->opcode == ZEND_YIELD_FROM))) {
-		generator = zend_generator_get_current(orig_generator);
-		goto try_again;
-	}
-
 	orig_generator->flags &= ~ZEND_GENERATOR_DO_INIT;
 }
 /* }}} */
@@ -887,9 +860,7 @@ ZEND_METHOD(Generator, current)
 
 	root = zend_generator_get_current(generator);
 	if (EXPECTED(generator->execute_data != NULL && Z_TYPE(root->value) != IS_UNDEF)) {
-		zval *value = &root->value;
-
-		ZVAL_COPY_DEREF(return_value, value);
+		RETURN_COPY_DEREF(&root->value);
 	}
 }
 /* }}} */
@@ -907,9 +878,7 @@ ZEND_METHOD(Generator, key)
 
 	root = zend_generator_get_current(generator);
 	if (EXPECTED(generator->execute_data != NULL && Z_TYPE(root->key) != IS_UNDEF)) {
-		zval *key = &root->key;
-
-		ZVAL_COPY_DEREF(return_value, key);
+		RETURN_COPY_DEREF(&root->key);
 	}
 }
 /* }}} */
@@ -958,9 +927,7 @@ ZEND_METHOD(Generator, send)
 
 	root = zend_generator_get_current(generator);
 	if (EXPECTED(generator->execute_data)) {
-		zval *value = &root->value;
-
-		ZVAL_COPY_DEREF(return_value, value);
+		RETURN_COPY_DEREF(&root->value);
 	}
 }
 /* }}} */
@@ -990,9 +957,7 @@ ZEND_METHOD(Generator, throw)
 
 		root = zend_generator_get_current(generator);
 		if (generator->execute_data) {
-			zval *value = &root->value;
-
-			ZVAL_COPY_DEREF(return_value, value);
+			RETURN_COPY_DEREF(&root->value);
 		}
 	} else {
 		/* If the generator is already closed throw the exception in the
@@ -1142,17 +1107,9 @@ zend_object_iterator *zend_generator_get_iterator(zend_class_entry *ce, zval *ob
 
 void zend_register_generator_ce(void) /* {{{ */
 {
-	zend_class_entry ce;
-
-	INIT_CLASS_ENTRY(ce, "Generator", class_Generator_methods);
-	zend_ce_generator = zend_register_internal_class(&ce);
-	zend_ce_generator->ce_flags |= ZEND_ACC_FINAL | ZEND_ACC_NO_DYNAMIC_PROPERTIES;
+	zend_ce_generator = register_class_Generator(zend_ce_iterator);
 	zend_ce_generator->create_object = zend_generator_create;
-	zend_ce_generator->serialize = zend_class_serialize_deny;
-	zend_ce_generator->unserialize = zend_class_unserialize_deny;
-
 	/* get_iterator has to be assigned *after* implementing the interface */
-	zend_class_implements(zend_ce_generator, 1, zend_ce_iterator);
 	zend_ce_generator->get_iterator = zend_generator_get_iterator;
 
 	memcpy(&zend_generator_handlers, &std_object_handlers, sizeof(zend_object_handlers));
@@ -1162,7 +1119,6 @@ void zend_register_generator_ce(void) /* {{{ */
 	zend_generator_handlers.clone_obj = NULL;
 	zend_generator_handlers.get_constructor = zend_generator_get_constructor;
 
-	INIT_CLASS_ENTRY(ce, "ClosedGeneratorException", NULL);
-	zend_ce_ClosedGeneratorException = zend_register_internal_class_ex(&ce, zend_ce_exception);
+	zend_ce_ClosedGeneratorException = register_class_ClosedGeneratorException(zend_ce_exception);
 }
 /* }}} */
