@@ -1,13 +1,79 @@
 <?php
 
-/** @generate-function-entries */
+/** @generate-class-entries */
 
 final class mysqli_driver
 {
+    /** @readonly */
+    public string $client_info;
+
+    /** @readonly */
+    public int $client_version;
+
+    /** @readonly */
+    public int $driver_version;
+
+    public bool $reconnect = false;
+
+    public int $report_mode = 0;
 }
 
 class mysqli
 {
+    /** @link mysqli.affected-rows */
+    public int|string $affected_rows;
+
+    /** @link mysqli.get-client-info */
+    public string $client_info;
+
+    /** @link mysqli.get-client-version */
+    public int $client_version;
+
+    /** @link mysqli.connect-errno */
+    public int $connect_errno;
+
+    /** @link mysqli.connect-error */
+    public ?string $connect_error;
+
+    /** @link mysqli.errno */
+    public int $errno;
+
+    /** @link mysqli.error */
+    public string $error;
+
+    /** @link mysqli.error-list */
+    public array $error_list;
+
+    /** @link mysqli.field-count */
+    public int $field_count;
+
+    /** @link mysqli.get-host-info */
+    public string $host_info;
+
+    /** @link mysqli.info */
+    public ?string $info;
+
+    /** @link mysqli.insert-id */
+    public int|string $insert_id;
+
+    /** @link mysqli.get-server-info */
+    public string $server_info;
+
+    /** @link mysqli.get-server-version */
+    public int $server_version;
+
+    /** @link mysqli.sqlstate */
+    public string $sqlstate;
+
+    /** @link mysqli.get-proto-info */
+    public int $protocol_version;
+
+    /** @link mysqli.thread-id */
+    public int $thread_id;
+
+    /** @link mysqli.warning-count */
+    public int $warning_count;
+
     public function __construct(
         ?string $hostname = null,
         ?string $username = null,
@@ -18,43 +84,44 @@ class mysqli
     ) {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_autocommit
      */
-    public function autocommit(bool $enable) {}
+    public function autocommit(bool $enable): bool {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_begin_transaction
      */
-    public function begin_transaction(int $flags = 0, ?string $name = null) {}
+    public function begin_transaction(int $flags = 0, ?string $name = null): bool {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_change_user
      */
-    public function change_user(string $username, string $password, ?string $database) {}
+    public function change_user(string $username, string $password, ?string $database): bool {}
 
     /**
-     * @return string
+     * @tentative-return-type
      * @alias mysqli_character_set_name
      */
-    public function character_set_name() {}
+    public function character_set_name(): string {}
 
     /**
-     * @return bool
+     * @return true
      * @alias mysqli_close
+     * @no-verify
      */
-    public function close() {}
+    public function close() {} // TODO make return type void
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_commit
      */
-    public function commit(int $flags = 0, ?string $name = null) {}
+    public function commit(int $flags = 0, ?string $name = null): bool {}
 
     /**
-     * @return bool|null
+     * @tentative-return-type
      * @alias mysqli_connect
      * @no-verify
      */
@@ -65,110 +132,113 @@ class mysqli
         ?string $database = null,
         ?int $port = null,
         ?string $socket = null
-    ) {}
+    ): bool {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_dump_debug_info
      */
-    public function dump_debug_info() {}
+    public function dump_debug_info(): bool {}
 
     /**
-     * @return bool
+     * @return true
      * @alias mysqli_debug
      * @no-verify Should really be a static method
      */
-    public function debug(string $options) {}
+    public function debug(string $options) {} // TODO make return type void
 
     /**
-     * @return object|null
+     * @tentative-return-type
      * @alias mysqli_get_charset
      */
-    public function get_charset() {}
+    public function get_charset(): ?object {}
 
     /**
-     * @return string
+     * @tentative-return-type
      * @alias mysqli_get_client_info
+     * @deprecated 8.1.0
      */
-    public function get_client_info() {}
+    public function get_client_info(): string {}
 
 #if defined(MYSQLI_USE_MYSQLND)
     /**
-     * @return array
+     * @return array<string, mixed>
+     * @tentative-return-type
      * @alias mysqli_get_connection_stats
      */
-    public function get_connection_stats() {}
+    public function get_connection_stats(): array {}
 #endif
 
     /**
-     * @return string
+     * @tentative-return-type
      * @alias mysqli_get_server_info
      */
-    public function get_server_info() {}
+    public function get_server_info(): string {}
 
     /**
-     * @return mysqli_warning|false
+     * @tentative-return-type
      * @alias mysqli_get_warnings
      */
-    public function get_warnings() {}
+    public function get_warnings(): mysqli_warning|false {}
 
     /**
-     * @return null|false
-     */
+     * @deprecated
+     * @return bool|null
+     * */
     public function init() {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_kill
      */
-    public function kill(int $process_id) {}
+    public function kill(int $process_id): bool {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_multi_query
      */
-    public function multi_query(string $query) {}
+    public function multi_query(string $query): bool {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_more_results
      */
-    public function more_results() {}
+    public function more_results(): bool {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_next_result
      */
-    public function next_result() {}
+    public function next_result(): bool {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_ping
      */
-    public function ping() {}
+    public function ping(): bool {}
 
 #if defined(MYSQLI_USE_MYSQLND)
     /**
-     * @return int|false
+     * @tentative-return-type
      * @alias mysqli_poll
      */
-    public static function poll(?array &$read, ?array &$error, array &$reject, int $seconds, int $microseconds = 0) {}
+    public static function poll(?array &$read, ?array &$error, array &$reject, int $seconds, int $microseconds = 0): int|false {}
 #endif
 
     /**
-     * @return mysqli_stmt|false
+     * @tentative-return-type
      * @alias mysqli_prepare
      */
-    public function prepare(string $query) {}
+    public function prepare(string $query): mysqli_stmt|false {}
 
     /**
-     * @return mysqli_result|bool
+     * @tentative-return-type
      * @alias mysqli_query
      */
-    public function query(string $query, int $result_mode = MYSQLI_STORE_RESULT) {}
+    public function query(string $query, int $result_mode = MYSQLI_STORE_RESULT): mysqli_result|bool {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_real_connect
      */
     public function real_connect(
@@ -179,81 +249,82 @@ class mysqli
         ?int $port = null,
         ?string $socket = null,
         int $flags = 0
-    ) {}
+    ): bool {}
 
     /**
-     * @return string
+     * @tentative-return-type
      * @alias mysqli_real_escape_string
      */
-    public function real_escape_string(string $string) {}
+    public function real_escape_string(string $string): string {}
 
 #if defined(MYSQLI_USE_MYSQLND)
     /**
-     * @return mysqli_result|bool
+     * @tentative-return-type
      * @alias mysqli_reap_async_query
      */
-    public function reap_async_query() {}
+    public function reap_async_query(): mysqli_result|bool {}
 #endif
 
     /**
-     * @return string
+     * @tentative-return-type
      * @alias mysqli_real_escape_string
      */
-    public function escape_string(string $string) {}
+    public function escape_string(string $string): string {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_real_query
      */
-    public function real_query(string $query) {}
+    public function real_query(string $query): bool {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_release_savepoint
      */
-    public function release_savepoint(string $name) {}
+    public function release_savepoint(string $name): bool {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_rollback
      */
-    public function rollback(int $flags = 0, ?string $name = null) {}
+    public function rollback(int $flags = 0, ?string $name = null): bool {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_savepoint
      */
-    public function savepoint(string $name) {}
+    public function savepoint(string $name): bool {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_select_db
      */
-    public function select_db(string $database) {}
+    public function select_db(string $database): bool {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_set_charset
      */
-    public function set_charset(string $charset) {}
+    public function set_charset(string $charset): bool {}
 
     /**
      * @param string|int $value
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_options
      */
-    public function options(int $option, $value) {}
+    public function options(int $option, $value): bool {}
 
     /**
      * @param string|int $value
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_options
      */
-    public function set_opt(int $option, $value) {}
+    public function set_opt(int $option, $value): bool {}
 
     /**
-     * @return bool
+     * @return true
      * @alias mysqli_ssl_set
+     * @no-verify
      */
     public function ssl_set(
         ?string $key,
@@ -261,257 +332,313 @@ class mysqli
         ?string $ca_certificate,
         ?string $ca_path,
         ?string $cipher_algos
-    ) {}
+    ) {} // TODO make return type void
 
     /**
-     * @return string|false
+     * @tentative-return-type
      * @alias mysqli_stat
      */
-    public function stat() {}
+    public function stat(): string|false {}
 
     /**
-     * @return mysqli_stmt|false
+     * @tentative-return-type
      * @alias mysqli_stmt_init
      */
-    public function stmt_init() {}
+    public function stmt_init(): mysqli_stmt|false {}
 
     /**
-     * @return mysqli_result|false
+     * @tentative-return-type
      * @alias mysqli_store_result
      */
-    public function store_result(int $mode = 0) {}
+    public function store_result(int $mode = 0): mysqli_result|false {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_thread_safe
      */
-    public function thread_safe() {}
+    public function thread_safe(): bool {}
 
     /**
-     * @return mysqli_result|false
+     * @tentative-return-type
      * @alias mysqli_use_result
      */
-    public function use_result() {}
+    public function use_result(): mysqli_result|false {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_refresh
      */
-    public function refresh(int $flags) {}
+    public function refresh(int $flags): bool {}
 }
 
 class mysqli_result implements IteratorAggregate
 {
+    /** @link mysqli-result.current-field */
+    public int $current_field;
+
+    /** @link mysqli-result.field-count */
+    public int $field_count;
+
+    /** @link mysqli-result.lengths */
+    public ?array $lengths;
+
+    /** @link mysqli-result.num-rows */
+    public int|string $num_rows;
+
+    public int $type;
+
     public function __construct(mysqli $mysql, int $result_mode = MYSQLI_STORE_RESULT) {}
 
     /**
-     * @return void
+     * @tentative-return-type
      * @alias mysqli_free_result
      */
-    public function close() {}
+    public function close(): void {}
 
     /**
-     * @return void
+     * @tentative-return-type
      * @alias mysqli_free_result
      */
-    public function free() {}
+    public function free(): void {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_data_seek
      */
-    public function data_seek(int $offset) {}
+    public function data_seek(int $offset): bool {}
 
     /**
-     * @return object|false
+     * @tentative-return-type
      * @alias mysqli_fetch_field
      */
-    public function fetch_field() {}
+    public function fetch_field(): object|false {}
 
     /**
-     * @return array
+     * @return array<int, object>
+     * @tentative-return-type
      * @alias mysqli_fetch_fields
      */
-    public function fetch_fields() {}
+    public function fetch_fields(): array {}
 
     /**
-     * @return object|false
+     * @tentative-return-type
      * @alias mysqli_fetch_field_direct
      */
-    public function fetch_field_direct(int $index) {}
+    public function fetch_field_direct(int $index): object|false {}
 
-#if defined(MYSQLI_USE_MYSQLND)
     /**
-     * @return array
+     * @return array<int|string, mixed>
+     * @tentative-return-type
      * @alias mysqli_fetch_all
      */
-    public function fetch_all(int $mode = MYSQLI_NUM) {}
-#endif
+    public function fetch_all(int $mode = MYSQLI_NUM): array {}
 
     /**
-     * @return array|null|false
+     * @return array<int|string, mixed>|null|false
+     * @tentative-return-type
      * @alias mysqli_fetch_array
      */
-    public function fetch_array(int $mode = MYSQLI_BOTH) {}
+    public function fetch_array(int $mode = MYSQLI_BOTH): array|null|false {}
 
     /**
-     * @return array|null|false
+     * @return array<int|string, mixed>|null|false
+     * @tentative-return-type
      * @alias mysqli_fetch_assoc
      */
-    public function fetch_assoc() {}
+    public function fetch_assoc(): array|null|false {}
 
     /**
-     * @return object|null|false
+     * @tentative-return-type
      * @alias mysqli_fetch_object
      */
-    public function fetch_object(string $class = "stdClass", array $constructor_args = []) {}
+    public function fetch_object(string $class = "stdClass", array $constructor_args = []): object|null|false {}
 
     /**
-     * @return array|null|false
+     * @return array<int, mixed>|null|false
+     * @tentative-return-type
      * @alias mysqli_fetch_row
      */
-    public function fetch_row() {}
+    public function fetch_row(): array|null|false {}
+
+    /** @alias mysqli_fetch_column */
+    public function fetch_column(int $column = 0): null|int|float|string|false {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_field_seek
      */
-    public function field_seek(int $index) {}
+    public function field_seek(int $index): bool {}
 
     /**
-     * @return void
+     * @tentative-return-type
      * @alias mysqli_free_result
      */
-    public function free_result() {}
+    public function free_result(): void {}
 
-    public function getIterator(): Iterator;
+    public function getIterator(): Iterator {}
 }
 
 class mysqli_stmt
 {
+    /** @link mysqli-stmt.affected-rows */
+    public int|string $affected_rows;
+
+    /** @link mysqli-stmt.insert-id */
+    public int|string $insert_id;
+
+    /** @link mysqli-stmt.num-rows */
+    public int|string $num_rows;
+
+    /** @link mysqli-stmt.param-count */
+    public int $param_count;
+
+    /** @link mysqli-stmt.field-count */
+    public int $field_count;
+
+    /** @link mysqli-stmt.errno */
+    public int $errno;
+
+    /** @link mysqli-stmt.error */
+    public string $error;
+
+    /** @link mysqli-stmt.error-list */
+    public array $error_list;
+
+    /** @link mysqli-stmt.sqlstate */
+    public string $sqlstate;
+
+    public int $id;
+
     public function __construct(mysqli $mysql, ?string $query = null) {}
 
     /**
-     * @return int
+     * @tentative-return-type
      * @alias mysqli_stmt_attr_get
      */
-    public function attr_get(int $attribute) {}
+    public function attr_get(int $attribute): int {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_stmt_attr_set
      */
-    public function attr_set(int $attribute, int $value) {}
+    public function attr_set(int $attribute, int $value): bool {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_stmt_bind_param
      */
-    public function bind_param(string $types, mixed &...$vars) {}
+    public function bind_param(string $types, mixed &...$vars): bool {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_stmt_bind_result
      */
-    public function bind_result(mixed &...$vars) {}
+    public function bind_result(mixed &...$vars): bool {}
 
     /**
-     * @return bool
+     * @return true
      * @alias mysqli_stmt_close
+     * @no-verify
      */
-    public function close() {}
+    public function close() {} // TODO make return type void
 
     /**
-     * @return void
+     * @tentative-return-type
      * @alias mysqli_stmt_data_seek
      */
-    public function data_seek(int $offset) {}
+    public function data_seek(int $offset): void {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_stmt_execute
      */
-    public function execute() {}
+    public function execute(?array $params = null): bool {}
 
     /**
-     * @return bool|null
+     * @tentative-return-type
      * @alias mysqli_stmt_fetch
      */
-    public function fetch() {}
+    public function fetch(): ?bool {}
 
     /**
-     * @return mysqli_warning|false
+     * @tentative-return-type
      * @alias mysqli_stmt_get_warnings
      */
-    public function get_warnings() {}
+    public function get_warnings(): mysqli_warning|false {}
 
     /**
-     * @return mysqli_result|false
+     * @tentative-return-type
      * @alias mysqli_stmt_result_metadata
      */
-    public function result_metadata() {}
+    public function result_metadata(): mysqli_result|false {}
 
 #if defined(MYSQLI_USE_MYSQLND)
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_stmt_more_results
      */
-    public function more_results() {}
-
-    /**
-     * @return bool
-     * @alias mysqli_stmt_next_result
-     */
-    public function next_result() {}
+    public function more_results(): bool {}
 #endif
 
     /**
-     * @return int|string
+     * @tentative-return-type
+     * @alias mysqli_stmt_next_result
+     */
+    public function next_result(): bool {}
+
+    /**
+     * @tentative-return-type
      * @alias mysqli_stmt_num_rows
      */
-    public function num_rows() {}
+    public function num_rows(): int|string {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_stmt_send_long_data
      */
-    public function send_long_data(int $param_num, string $data) {}
+    public function send_long_data(int $param_num, string $data): bool {}
 
     /**
-     * @return void
+     * @tentative-return-type
      * @alias mysqli_stmt_free_result
      */
-    public function free_result() {}
+    public function free_result(): void {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_stmt_reset
      */
-    public function reset() {}
+    public function reset(): bool {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_stmt_prepare
      */
-    public function prepare(string $query) {}
+    public function prepare(string $query): bool {}
 
     /**
-     * @return bool
+     * @tentative-return-type
      * @alias mysqli_stmt_store_result
      */
-    public function store_result() {}
+    public function store_result(): bool {}
 
 #if defined(MYSQLI_USE_MYSQLND)
     /**
-     * @return mysqli_result|false
+     * @tentative-return-type
      * @alias mysqli_stmt_get_result
      */
-    public function get_result() {}
+    public function get_result(): mysqli_result|false {}
 #endif
 }
 
 final class mysqli_warning
 {
+    public string $message;
+
+    public string $sqlstate;
+
+    public int $errno;
+
     private function __construct() {}
 
     public function next(): bool {}
@@ -519,8 +646,12 @@ final class mysqli_warning
 
 final class mysqli_sql_exception extends RuntimeException
 {
+    protected string $sqlstate = "00000";
+
+    public function getSqlState(): string {}
 }
 
+/** @refcount 1 */
 function mysqli_affected_rows(mysqli $mysql): int|string {}
 
 function mysqli_autocommit(mysqli $mysql, bool $enable): bool {}
@@ -529,12 +660,15 @@ function mysqli_begin_transaction(mysqli $mysql, int $flags = 0, ?string $name =
 
 function mysqli_change_user(mysqli $mysql, string $username, string $password, ?string $database): bool {}
 
+/** @refcount 1 */
 function mysqli_character_set_name(mysqli $mysql): string {}
 
-function mysqli_close(mysqli $mysql): bool {}
+/** @return true */
+function mysqli_close(mysqli $mysql): bool {} // TODO make return type void
 
 function mysqli_commit(mysqli $mysql, int $flags = 0, ?string $name = null): bool {}
 
+/** @refcount 1 */
 function mysqli_connect(
     ?string $hostname = null,
     ?string $username = null,
@@ -546,45 +680,79 @@ function mysqli_connect(
 
 function mysqli_connect_errno(): int {}
 
+/** @refcount 1 */
 function mysqli_connect_error(): ?string {}
 
 function mysqli_data_seek(mysqli_result $result, int $offset): bool {}
 
 function mysqli_dump_debug_info(mysqli $mysql): bool {}
 
-function mysqli_debug(string $options): bool {}
+/** @return true */
+function mysqli_debug(string $options): bool {} // TODO make return type void
 
 function mysqli_errno(mysqli $mysql): int {}
 
+/** @refcount 1 */
 function mysqli_error(mysqli $mysql): string {}
 
+/**
+ * @return array<int, array>
+ * @refcount 1
+ */
 function mysqli_error_list(mysqli $mysql): array {}
 
-function mysqli_stmt_execute(mysqli_stmt $statement): bool {}
+function mysqli_stmt_execute(mysqli_stmt $statement, ?array $params = null): bool {}
 
 /** @alias mysqli_stmt_execute */
-function mysqli_execute(mysqli_stmt $statement): bool {}
+function mysqli_execute(mysqli_stmt $statement, ?array $params = null): bool {}
 
+/** @refcount 1 */
 function mysqli_fetch_field(mysqli_result $result): object|false {}
 
+/**
+ * @return array<int, object>
+ * @refcount 1
+ */
 function mysqli_fetch_fields(mysqli_result $result): array {}
 
+/** @refcount 1 */
 function mysqli_fetch_field_direct(mysqli_result $result, int $index): object|false {}
 
+/**
+ * @return array<int, int>|false
+ * @refcount 1
+ */
 function mysqli_fetch_lengths(mysqli_result $result): array|false {}
 
-#if defined(MYSQLI_USE_MYSQLND)
+/**
+ * @return array<int|string, mixed>
+ * @refcount 1
+ */
 function mysqli_fetch_all(mysqli_result $result, int $mode = MYSQLI_NUM): array {}
-#endif
 
+/**
+ * @return array<int|string, mixed>|null|false
+ * @refcount 1
+ */
 function mysqli_fetch_array(mysqli_result $result, int $mode = MYSQLI_BOTH): array|null|false {}
 
+/**
+ * @return array<int|string, mixed>|null|false
+ * @refcount 1
+ */
 function mysqli_fetch_assoc(mysqli_result $result): array|null|false {}
 
+/** @refcount 1 */
 function mysqli_fetch_object(mysqli_result $result, string $class = "stdClass", array $constructor_args = []): object|null|false {}
 
+/**
+ * @return array<int, mixed>|null|false
+ * @refcount 1
+ */
 function mysqli_fetch_row(mysqli_result $result): array|null|false {}
 
+function mysqli_fetch_column(mysqli_result $result, int $column = 0): null|int|float|string|false {}
+
 function mysqli_field_count(mysqli $mysql): int {}
 
 function mysqli_field_seek(mysqli_result $result, int $index): bool {}
@@ -594,33 +762,53 @@ function mysqli_field_tell(mysqli_result $result): int {}
 function mysqli_free_result(mysqli_result $result): void {}
 
 #if defined(MYSQLI_USE_MYSQLND)
+/**
+ * @return array<string, mixed>
+ * @refcount 1
+ */
 function mysqli_get_connection_stats(mysqli $mysql): array {}
 
+/**
+ * @return array<string, string>
+ * @refcount 1
+ */
 function mysqli_get_client_stats(): array {}
 #endif
 
+/** @refcount 1 */
 function mysqli_get_charset(mysqli $mysql): ?object {}
 
+/** @refcount 1 */
 function mysqli_get_client_info(?mysqli $mysql = null): string {}
 
 function mysqli_get_client_version(): int {}
 
+/**
+ * @return array<string, int>
+ * @refcount 1
+ */
 function mysqli_get_links_stats(): array {}
 
+/** @refcount 1 */
 function mysqli_get_host_info(mysqli $mysql): string {}
 
 function mysqli_get_proto_info(mysqli $mysql): int {}
 
+/** @refcount 1 */
 function mysqli_get_server_info(mysqli $mysql): string {}
 
 function mysqli_get_server_version(mysqli $mysql): int {}
 
+/** @refcount 1 */
 function mysqli_get_warnings(mysqli $mysql): mysqli_warning|false {}
 
+/** @refcount 1 */
 function mysqli_init(): mysqli|false {}
 
+/** @refcount 1 */
 function mysqli_info(mysqli $mysql): ?string {}
 
+/** @refcount 1 */
 function mysqli_insert_id(mysqli $mysql): int|string {}
 
 function mysqli_kill(mysqli $mysql, int $process_id): bool {}
@@ -633,6 +821,7 @@ function mysqli_next_result(mysqli $mysql): bool {}
 
 function mysqli_num_fields(mysqli_result $result): int {}
 
+/** @refcount 1 */
 function mysqli_num_rows(mysqli_result $result): int|string {}
 
 /** @param string|int $value */
@@ -650,10 +839,12 @@ function mysqli_ping(mysqli $mysql): bool {}
 function mysqli_poll(?array &$read, ?array &$error, array &$reject, int $seconds, int $microseconds = 0): int|false {}
 #endif
 
+/** @refcount 1 */
 function mysqli_prepare(mysqli $mysql, string $query): mysqli_stmt|false {}
 
 function mysqli_report(int $flags): bool {}
 
+/** @refcount 1 */
 function mysqli_query(mysqli $mysql, string $query, int $result_mode = MYSQLI_STORE_RESULT): mysqli_result|bool {}
 
 function mysqli_real_connect(
@@ -667,6 +858,7 @@ function mysqli_real_connect(
     int $flags = 0
 ): bool {}
 
+/** @refcount 1 */
 function mysqli_real_escape_string(mysqli $mysql, string $string): string {}
 
 /** @alias mysqli_real_escape_string */
@@ -675,6 +867,7 @@ function mysqli_escape_string(mysqli $mysql, string $string): string {}
 function mysqli_real_query(mysqli $mysql, string $query): bool {}
 
 #if defined(MYSQLI_USE_MYSQLND)
+/** @refcount 1 */
 function mysqli_reap_async_query(mysqli $mysql): mysqli_result|bool {}
 #endif
 
@@ -688,6 +881,7 @@ function mysqli_select_db(mysqli $mysql, string $database): bool {}
 
 function mysqli_set_charset(mysqli $mysql, string $charset): bool {}
 
+/** @refcount 1 */
 function mysqli_stmt_affected_rows(mysqli_stmt $statement): int|string {}
 
 function mysqli_stmt_attr_get(mysqli_stmt $statement, int $attribute): int {}
@@ -698,14 +892,20 @@ function mysqli_stmt_bind_param(mysqli_stmt $statement, string $types, mixed &..
 
 function mysqli_stmt_bind_result(mysqli_stmt $statement, mixed &...$vars): bool {}
 
+/** @return true */
 function mysqli_stmt_close(mysqli_stmt $statement): bool {}
 
 function mysqli_stmt_data_seek(mysqli_stmt $statement, int $offset): void {}
 
 function mysqli_stmt_errno(mysqli_stmt $statement): int {}
 
+/** @refcount 1 */
 function mysqli_stmt_error(mysqli_stmt $statement): string {}
 
+/**
+ * @return array<int, array>
+ * @refcount 1
+ */
 function mysqli_stmt_error_list(mysqli_stmt $statement): array {}
 
 function mysqli_stmt_fetch(mysqli_stmt $statement): ?bool {}
@@ -715,21 +915,26 @@ function mysqli_stmt_field_count(mysqli_stmt $statement): int {}
 function mysqli_stmt_free_result(mysqli_stmt $statement): void {}
 
 #if defined(MYSQLI_USE_MYSQLND)
+/** @refcount 1 */
 function mysqli_stmt_get_result(mysqli_stmt $statement): mysqli_result|false {}
 #endif
 
+/** @refcount 1 */
 function mysqli_stmt_get_warnings(mysqli_stmt $statement): mysqli_warning|false {}
 
+/** @refcount 1 */
 function mysqli_stmt_init(mysqli $mysql): mysqli_stmt|false {}
 
+/** @refcount 1 */
 function mysqli_stmt_insert_id(mysqli_stmt $statement): int|string {}
 
 #if defined(MYSQLI_USE_MYSQLND)
 function mysqli_stmt_more_results(mysqli_stmt $statement): bool {}
-
-function mysqli_stmt_next_result(mysqli_stmt $statement): bool {}
 #endif
 
+function mysqli_stmt_next_result(mysqli_stmt $statement): bool {}
+
+/** @refcount 1 */
 function mysqli_stmt_num_rows(mysqli_stmt $statement): int|string {}
 
 function mysqli_stmt_param_count(mysqli_stmt $statement): int {}
@@ -738,16 +943,20 @@ function mysqli_stmt_prepare(mysqli_stmt $statement, string $query): bool {}
 
 function mysqli_stmt_reset(mysqli_stmt $statement): bool {}
 
+/** @refcount 1 */
 function mysqli_stmt_result_metadata(mysqli_stmt $statement): mysqli_result|false {}
 
 function mysqli_stmt_send_long_data(mysqli_stmt $statement, int $param_num, string $data): bool {}
 
 function mysqli_stmt_store_result(mysqli_stmt $statement): bool {}
 
+/** @refcount 1 */
 function mysqli_stmt_sqlstate(mysqli_stmt $statement): string {}
 
+/** @refcount 1 */
 function mysqli_sqlstate(mysqli $mysql): string {}
 
+/** @return true */
 function mysqli_ssl_set(
     mysqli $mysql,
     ?string $key,
@@ -755,16 +964,19 @@ function mysqli_ssl_set(
     ?string $ca_certificate,
     ?string $ca_path,
     ?string $cipher_algos
-): bool {}
+): bool {} // TODO make return type void
 
+/** @refcount 1 */
 function mysqli_stat(mysqli $mysql): string|false {}
 
+/** @refcount 1 */
 function mysqli_store_result(mysqli $mysql, int $mode = 0): mysqli_result|false {}
 
 function mysqli_thread_id(mysqli $mysql): int {}
 
 function mysqli_thread_safe(): bool {}
 
+/** @refcount 1 */
 function mysqli_use_result(mysqli $mysql): mysqli_result|false {}
 
 function mysqli_warning_count(mysqli $mysql): int {}
