@@ -5,7 +5,7 @@
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | https://www.php.net/license/3_01.txt                                 |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -47,11 +47,7 @@
 #endif
 
 #ifndef NSIG
-# ifdef SIGRTMAX
-#  define NSIG (SIGRTMAX + 1)
-# else
-#  define NSIG 32
-# endif
+# define NSIG 32
 #endif
 
 ZEND_DECLARE_MODULE_GLOBALS(pcntl)
@@ -89,7 +85,8 @@ static void pcntl_siginfo_to_zval(int, siginfo_t*, zval*);
 #else
 static void pcntl_signal_handler(int);
 #endif
-static void pcntl_signal_dispatch();
+static void pcntl_signal_dispatch(void);
+static void pcntl_signal_dispatch_tick_function(int dummy_int, void *dummy_pointer);
 static void pcntl_interrupt_function(zend_execute_data *execute_data);
 
 void php_register_signal_constants(INIT_FUNC_ARGS)
@@ -151,9 +148,7 @@ void php_register_signal_constants(INIT_FUNC_ARGS)
 #ifdef SIGPOLL
 	REGISTER_LONG_CONSTANT("SIGPOLL",  (zend_long) SIGPOLL, CONST_CS | CONST_PERSISTENT);
 #endif
-#ifdef SIGIO
 	REGISTER_LONG_CONSTANT("SIGIO",    (zend_long) SIGIO, CONST_CS | CONST_PERSISTENT);
-#endif
 #ifdef SIGPWR
 	REGISTER_LONG_CONSTANT("SIGPWR",   (zend_long) SIGPWR, CONST_CS | CONST_PERSISTENT);
 #endif
@@ -172,6 +167,10 @@ void php_register_signal_constants(INIT_FUNC_ARGS)
 	REGISTER_LONG_CONSTANT("PRIO_PGRP", PRIO_PGRP, CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("PRIO_USER", PRIO_USER, CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("PRIO_PROCESS", PRIO_PROCESS, CONST_CS | CONST_PERSISTENT);
+#if defined(PRIO_DARWIN_BG)
+	REGISTER_LONG_CONSTANT("PRIO_DARWIN_BG", PRIO_DARWIN_BG, CONST_CS | CONST_PERSISTENT);
+	REGISTER_LONG_CONSTANT("PRIO_DARWIN_THREAD", PRIO_DARWIN_THREAD, CONST_CS | CONST_PERSISTENT);
+#endif
 #endif
 
 	/* {{{ "how" argument for sigprocmask */
@@ -341,6 +340,30 @@ void php_register_signal_constants(INIT_FUNC_ARGS)
 	REGISTER_LONG_CONSTANT("CLONE_NEWCGROUP",	CLONE_NEWCGROUP, CONST_CS | CONST_PERSISTENT);
 #endif
 #endif
+
+#ifdef HAVE_RFORK
+#ifdef RFPROC
+	REGISTER_LONG_CONSTANT("RFPROC",	RFPROC, CONST_CS | CONST_PERSISTENT);
+#endif
+#ifdef RFNOWAIT
+	REGISTER_LONG_CONSTANT("RFNOWAIT",	RFNOWAIT, CONST_CS | CONST_PERSISTENT);
+#endif
+#ifdef RFCFDG
+	REGISTER_LONG_CONSTANT("RFCFDG",	RFCFDG, CONST_CS | CONST_PERSISTENT);
+#endif
+#ifdef RFFDG
+	REGISTER_LONG_CONSTANT("RFFDG",	RFFDG, CONST_CS | CONST_PERSISTENT);
+#endif
+#ifdef RFLINUXTHPN
+	REGISTER_LONG_CONSTANT("RFLINUXTHPN",	RFLINUXTHPN, CONST_CS | CONST_PERSISTENT);
+#endif
+#ifdef RFTSIGZMB
+	REGISTER_LONG_CONSTANT("RFTSIGZMB",	RFTSIGZMB, CONST_CS | CONST_PERSISTENT);
+#endif
+#ifdef RFTHREAD
+	REGISTER_LONG_CONSTANT("RFTHREAD",	RFTHREAD, CONST_CS | CONST_PERSISTENT);
+#endif
+#endif
 }
 
 static void php_pcntl_register_errno_constants(INIT_FUNC_ARGS)
@@ -426,11 +449,19 @@ static PHP_GINIT_FUNCTION(pcntl)
 
 PHP_RINIT_FUNCTION(pcntl)
 {
-	php_add_tick_function(pcntl_signal_dispatch, NULL);
+	php_add_tick_function(pcntl_signal_dispatch_tick_function, NULL);
 	zend_hash_init(&PCNTL_G(php_signal_table), 16, NULL, ZVAL_PTR_DTOR, 0);
 	PCNTL_G(head) = PCNTL_G(tail) = PCNTL_G(spares) = NULL;
 	PCNTL_G(async_signals) = 0;
 	PCNTL_G(last_error) = 0;
+	PCNTL_G(num_signals) = NSIG;
+#ifdef SIGRTMAX
+	/* At least FreeBSD reports an incorrecrt NSIG that does not include realtime signals.
+	 * As SIGRTMAX may be a dynamic value, adjust the value in INIT. */
+	if (NSIG < SIGRTMAX + 1) {
+		PCNTL_G(num_signals) = SIGRTMAX + 1;
+	}
+#endif
 	return SUCCESS;
 }
 
@@ -888,8 +919,8 @@ PHP_FUNCTION(pcntl_signal)
 {
 	zval *handle;
 	zend_long signo;
-	zend_bool restart_syscalls = 1;
-	zend_bool restart_syscalls_is_null = 1;
+	bool restart_syscalls = 1;
+	bool restart_syscalls_is_null = 1;
 	char *error = NULL;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "lz|b!", &signo, &handle, &restart_syscalls, &restart_syscalls_is_null) == FAILURE) {
@@ -901,8 +932,8 @@ PHP_FUNCTION(pcntl_signal)
 		RETURN_THROWS();
 	}
 
-	if (signo >= NSIG) {
-		zend_argument_value_error(1, "must be less than %d", NSIG);
+	if (signo >= PCNTL_G(num_signals)) {
+		zend_argument_value_error(1, "must be less than %d", PCNTL_G(num_signals));
 		RETURN_THROWS();
 	}
 
@@ -910,7 +941,7 @@ PHP_FUNCTION(pcntl_signal)
 		/* since calling malloc() from within a signal handler is not portable,
 		 * pre-allocate a few records for recording signals */
 		int i;
-		for (i = 0; i < NSIG; i++) {
+		for (i = 0; i < PCNTL_G(num_signals); i++) {
 			struct php_pcntl_pending_signal *psig;
 
 			psig = emalloc(sizeof(*psig));
@@ -1038,7 +1069,7 @@ PHP_FUNCTION(pcntl_sigprocmask)
 			RETURN_THROWS();
 		}
 
-		for (signo = 1; signo < NSIG; ++signo) {
+		for (signo = 1; signo < PCNTL_G(num_signals); ++signo) {
 			if (sigismember(&oldset, signo) != 1) {
 				continue;
 			}
@@ -1162,7 +1193,7 @@ static void pcntl_siginfo_to_zval(int signo, siginfo_t *siginfo, zval *user_sigi
 			case SIGBUS:
 				add_assoc_double_ex(user_siginfo, "addr", sizeof("addr")-1, (zend_long)siginfo->si_addr);
 				break;
-#if defined(SIGPOLL) && !defined(__CYGWIN__)
+#ifdef SIGPOLL
 			case SIGPOLL:
 				add_assoc_long_ex(user_siginfo, "band", sizeof("band")-1, siginfo->si_band);
 # ifdef si_fd
@@ -1188,7 +1219,7 @@ PHP_FUNCTION(pcntl_getpriority)
 {
 	zend_long who = PRIO_PROCESS;
 	zend_long pid;
-	zend_bool pid_is_null = 1;
+	bool pid_is_null = 1;
 	int pri;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "|l!l", &pid, &pid_is_null, &who) == FAILURE) {
@@ -1227,7 +1258,7 @@ PHP_FUNCTION(pcntl_setpriority)
 {
 	zend_long who = PRIO_PROCESS;
 	zend_long pid;
-	zend_bool pid_is_null = 1;
+	bool pid_is_null = 1;
 	zend_long pri;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "l|l!l", &pri, &pid, &pid_is_null, &who) == FAILURE) {
@@ -1387,10 +1418,15 @@ void pcntl_signal_dispatch()
 	sigprocmask(SIG_SETMASK, &old_mask, NULL);
 }
 
+static void pcntl_signal_dispatch_tick_function(int dummy_int, void *dummy_pointer)
+{
+	return pcntl_signal_dispatch();
+}
+
 /* {{{ Enable/disable asynchronous signal handling and return the old setting. */
 PHP_FUNCTION(pcntl_async_signals)
 {
-	zend_bool on, on_is_null = 1;
+	bool on, on_is_null = 1;
 
 	if (zend_parse_parameters(ZEND_NUM_ARGS(), "|b!", &on, &on_is_null) == FAILURE) {
 		RETURN_THROWS();
@@ -1456,6 +1492,67 @@ PHP_FUNCTION(pcntl_unshare)
 /* }}} */
 #endif
 
+#ifdef HAVE_RFORK
+/* {{{ proto bool pcntl_rfork(int flags [, int signal])
+   More control over the process creation is given over fork/vfork. */
+PHP_FUNCTION(pcntl_rfork)
+{
+	zend_long flags;
+	zend_long csignal = 0;
+	pid_t pid;
+
+	ZEND_PARSE_PARAMETERS_START(1, 2)
+		Z_PARAM_LONG(flags)
+		Z_PARAM_OPTIONAL
+		Z_PARAM_LONG(csignal)
+	ZEND_PARSE_PARAMETERS_END();
+
+	/* This is a flag to use with great caution in general, preferably not within PHP */
+	if ((flags & RFMEM) != 0) {
+		zend_argument_value_error(1, "must not include RFMEM value, not allowed within this context");
+		RETURN_THROWS();
+	}
+
+	if ((flags & RFSIGSHARE) != 0) {
+		zend_argument_value_error(1, "must not include RFSIGSHARE value, not allowed within this context");
+		RETURN_THROWS();
+	}
+
+	if ((flags & (RFFDG | RFCFDG)) == (RFFDG | RFCFDG)) {
+		zend_argument_value_error(1, "must not include both RFFDG and RFCFDG, because these flags are mutually exclusive");
+		RETURN_THROWS();
+	}
+
+	/* A new pid is required */
+	if (!(flags & (RFPROC))) {
+		flags |= RFPROC;
+	}
+
+#ifdef RFTSIGZMB
+	if ((flags & RFTSIGZMB) != 0) {
+		flags |= RFTSIGFLAGS(csignal);
+	}
+#endif
+
+	pid = rfork(flags);
+
+	if (pid == -1) {
+		PCNTL_G(last_error) = errno;
+		switch (errno) {
+			case EAGAIN:
+			php_error_docref(NULL, E_WARNING, "Maximum process creations limit reached\n");
+		break;
+
+		default:
+			php_error_docref(NULL, E_WARNING, "Error %d", errno);
+		}
+	}
+
+	RETURN_LONG((zend_long) pid);
+}
+#endif
+/* }}} */
+
 static void pcntl_interrupt_function(zend_execute_data *execute_data)
 {
 	pcntl_signal_dispatch();
